# -*- coding: utf-8 -*-

from django.conf import settings as s

# URL
WIDGET_URL = getattr(s, 'ULOGIN_WIDGET_URL', 'http://ulogin.ru/js/widget.js')

# URL to get token
TOKEN_URL = getattr(s, 'ULOGIN_TOKEN_URL', 'http://ulogin.ru/token.php')

REDIRECT_URL = getattr(s, 'ULOGIN_REDIRECT_URL', None)

################################################################################
# Currently ULOGIN supports these fields                                       #
################################################################################
#
#    first_name
#    last_name
#    email
#    nickname
#    bdate      - user birthday in 'DD.MM.YYYY' format;
#    sex        - user gender in integer: 1 for "Female", 2 for Male;
#    photo      - Square avatar (up to 100x100);
#    photo_big  - Bigest photo provided by social network;
#    city
#    country
#
################################################################################
# Required fields

FIELDS = getattr(s, 'ULOGIN_FIELDS', ['email'])

# Optional fields
OPTIONAL = getattr(s, 'ULOGIN_OPTIONAL', [])

# Display type. The variants 'panel', 'small' and 'button' are available
DISPLAY = getattr(s, 'ULOGIN_DISPLAY', 'small')

# Featured providers
PROVIDERS = getattr(s, 'ULOGIN_PROVIDERS', 
                'vkontakte,facebook,twitter,google,livejournal'.split(','))

# Providers in dropdown list
HIDDEN = getattr(s, 'ULOGIN_HIDDEN',
                 'yandex,odnoklassniki,mailru,openid'.split(','))

# Callback function
CALLBACK = getattr(s, 'ULOGIN_CALLBACK', None)



# -*- coding: utf-8 -*-

from django import template
from django.core.urlresolvers import reverse as r
from django.contrib.auth import REDIRECT_FIELD_NAME as FLD
from django_ulogin import settings as s
import urllib

def get_redirect_url(request):
    if getattr(s, 'REDIRECT_URL'):
        return s.REDIRECT_URL

    # Current URL
    return_url = request.build_absolute_uri(request.path_info)

    # Hack the request.GET
    if FLD not in request.GET:
        get = request.GET.copy()
        get.update({ FLD: request.build_absolute_uri(request.path_info) })
        request.GET = get
    
    return urllib.quote(u"{request_url}?{query_string}".format(
        request_url  = request.build_absolute_uri(r('ulogin_postback')),
        query_string = urllib.unquote(request.GET.urlencode())
        ))

def ulogin_widget(context):
    """
    """
    return {
        'WIDGET_URL'   : s.WIDGET_URL,
        'CALLBACK'     : s.CALLBACK,
        'DISPLAY'      : s.DISPLAY,
        'REDIRECT_URL' : get_redirect_url(context['request']),
        'FIELDS'       : ','.join([f for f in s.FIELDS]),
        'OPTIONAL'     : ','.join([o for o in s.OPTIONAL]),
        'PROVIDERS'    : ','.join([p for p in s.PROVIDERS]),
        'HIDDEN'       : ','.join([h for h in s.HIDDEN]),
    }

register = template.Library()
register.inclusion_tag('django_ulogin/ulogin_widget.html',
                       takes_context=True)(ulogin_widget)



# -*- coding: utf-8 -*-

from django.conf.urls.defaults import url, patterns

urlpatterns = patterns('django_ulogin.views',
    url('^postback/$', 'postback', name='ulogin_postback'),
    url('^ulogin_xd.html$', 'ulogin_xd', name='ulogin_xd'),
)


# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django_ulogin import settings as s
import datetime

class ULoginUser(models.Model):
    user = models.ForeignKey(User,
        related_name = 'ulogin_users',
        verbose_name = _('user'))
    network = models.CharField(_('network'),
        db_index = True,
        max_length = 255,
        choices = [(n, n) for n in (s.PROVIDERS + s.OPTIONAL)])
    identity = models.URLField(_('identity'),
        db_index = True,
        max_length = 255)
    uid = models.CharField(_('uid'),
        db_index = True,
        max_length = 255)
    date_created = models.DateTimeField(_('date created'),
        editable = False,
        default = datetime.datetime.now)

    def __unicode__(self):
        return unicode(self.user)

    class Meta:
        verbose_name = _('ulogin user')
        verbose_name_plural = _('ulogin users')
        unique_together = [('network', 'uid')]



# -*- coding: utf-8 -*-

from django.conf.urls.defaults import *
from django.http import HttpResponse
from django.template import Context, Template

def index(request):
    template = Template("")
    context = Context({'request': request, 'user': request.user})
    return HttpResponse( template.render(context) )

urlpatterns = patterns('',
    url('^django_ulogin/', include('django_ulogin.urls')),
    url('^$', index)
)


# -*- coding: utf-8 -*-

from django.contrib.auth import login
from django.http import HttpResponseNotAllowed, HttpResponseBadRequest
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.models import User
from django.shortcuts import redirect
from django.views.decorators.csrf import csrf_exempt
from django.utils import simplejson
from django_ulogin import settings
from django_ulogin.models import ULoginUser
from django_ulogin.signals import assign
import requests
import uuid

try:
    from django.shortcuts import render
except ImportError:
    from django.views.generic.simple import direct_to_template as render

def ulogin_response(token, host):
    """
    """
    return simplejson.loads(requests.get(settings.TOKEN_URL, params={
                               'token' : token,
                               'host'  : host
                           }).content)

@csrf_exempt
def postback(request):
    """
    """
    if request.method != 'POST':
        return HttpResponseNotAllowed(['POST'])

    if 'token' not in request.POST:
        return HttpResponseBadRequest()

    response = ulogin_response(request.POST['token'], request.get_host())

    if 'error' in response:
        return render(request, 'django_ulogin/error.html', {'json': response})

    if request.user.is_authenticated():
        user = request.user
        registered = False
        ulogin = None

        if not request.user.ulogin_users.count():
            ulogin = ULoginUser.objects.create(network  = response['network'],
                                               uid      = response['uid'],
                                               identity = response['identity'],
                                               user     = user)
            registered = True
    # Not authenticated
    else:
        try:
            ulogin = ULoginUser.objects.get(network=response['network'],
                                            uid=response['uid'])
            registered = False
            user = ulogin.user

        except ULoginUser.DoesNotExist:
            user = User()
            user.username=uuid.uuid4().hex[:30]
            user.set_unusable_password()
            user.save()

            ulogin = ULoginUser.objects.create(network  = response['network'],
                                               uid      = response['uid'],
                                               identity = response['identity'],
                                               user     = user)
            registered = True

        # Authenticate user
        if not hasattr(user, 'backend'):
            user.backend = 'django.contrib.auth.backends.ModelBackend'

        login(request, user)

    # End of not authenticated
    assign.send(sender=ULoginUser, request=request, user=user,
                registered=registered,
                ulogin_user=ulogin, ulogin_data=response)

    return redirect(request.GET.get(REDIRECT_FIELD_NAME) or '/')

def ulogin_xd(request):
    """
    Document for avoid cross domain security policies
    """
    return render(request, 'django_ulogin/ulogin_xd.html')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import os
version = __import__('django_ulogin').get_version()
CLASSIFIERS=[
    'Development Status :: 4 - Beta',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Framework :: Django'
]

setup(
    name = 'django-ulogin',
    author = 'marazmiki',
    version = version,
    author_email = 'marazmiki@gmail.com',
    url = 'http://pypi.python.org/pypi/django-ulogin',
    download_url = 'http://bitbucket.org/marazmiki/django-ulogin/get/tip.zip',
    description = 'User social authentication with ulogin.ru service',
    long_description = open(os.path.join(os.path.dirname(__file__), 'README')).read(),
    license = 'MIT license',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    install_requires=[
        'Django>=1.2.5',
        'requests',
    ],
    packages=find_packages(exclude=['test_project', 'test_project.*']),
    include_package_data=True,
    zip_safe = False
)

